"""
Physical inventory list extension for fava.

Usage: 
YYYY-MM-DD custom "fava-extension" "fava_inventory" "[('Consignment', 'In-Transit', 'In-Stock'), 'inventory.beancount']"
"""
import re

from beancount.core.data import Commodity
from beancount.core.number import Decimal

from fava.ext import FavaExtensionBase
from fava.helpers import FavaAPIException
from fava.template_filters import cost_or_value

class InventoryList(FavaExtensionBase):  # pragma: no cover

    report_title = "Inventory"

    # returns ledger
    def get_ledger(self):
        return self.ledger

    # returns types for query_table headers
    def get_types(self):

        types = []

        types.append(("SKU", str(str)))
        types.append(("Name", str(str)))

        for account in self.config[0]:
            types.append((account, str(Decimal)))

        types.append(("Total", str(Decimal)))

        return types

    # returns contents of query_table
    def get_inventories(self):

        tree = self.ledger.root_tree
        commodities = self.ledger.all_entries_by_type.Commodity

        inventories = {}

        for commodity in commodities:
            if (self.config[1] is not None and re.compile("^.*" + self.config[1] +"$").match(commodity.meta['filename']) is not None):
                inventory = {}
                inventory['Name'] = commodity.meta['name']
                inventories[commodity.currency] = inventory

        for inventory_account in self._get_inventory_accounts(tree):
            label = inventory_account[0]
            account = inventory_account[1]
            for bal in account.balance:
                sku = bal[0]
                inventory = inventories.get(sku) if (inventories.get(sku) is not None) else {}
                inventory[label] = inventory[label] + account.balance[bal] if label in inventory else account.balance[bal]
                inventories[sku] = inventory

        return self._get_rows_from_inventories(inventories)

    # returns list of tuples (label, account_node)
    def _get_inventory_accounts(self, tree):
        selected_accounts = []
        for account in self.config[0]:
            for key in tree.keys():
                if (re.compile("^.*:" + account + "$").match(key) is not None):
                    selected_accounts.append((account, tree[key]))
                    break
        
        return selected_accounts

    # returns rows as results based on inventories dictionary
    def _get_rows_from_inventories(self, inventories):
        rows = []
        for inventory in inventories:
            row = {}
            row["SKU"] = inventory

            balances = inventories[inventory]
            total = 0
            for balance in balances:
                row[balance] = balances[balance]
                if (balance != "Name"):
                    total += row[balance]
            row["Total"] = total

            rows.append(row)

        return rows


from os import path
from setuptools import find_packages, setup

with open(path.join(path.dirname(__file__), 'README.md')) as readme:
    LONG_DESCRIPTION = readme.read()

setup(
    name='fava_inventory',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    description='Fava extension for beancount physical inventory balances',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/ROCHK/fava_inventory',
    author='Ronald Cheng',
    author_email='ROCHK@users.noreply.github.com',
    license='GPL-3.0',
    keywords='fava beancount inventory management',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'beancount>=2.2.3',
        'fava>=1.19',
        'argh>=0.23.3',
        'argcomplete>=1.11.1'
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Office/Business :: Financial :: Accounting',
    ],
)